#######################################################################################
############################### RENDER NODE MANAGER ###################################
#######################################################################################

# RNM.pyw is an open source network tool to help you manage your renderfarm nodes.
# It's designed to work with AFANASY, the Open Source Renderfarm Manager from CGRU. 
# This version is also made for Distributed Rendering with VRay For Maya.
# But you can change and associate any application you want. 
# This script works only on windows machines.
# You need Python 3 installed. 

# With RNM you can :
#   - Wake on lan or shutdown your clients (using WakeOnLan)
#   - Remote your client's desktop (using Anydesk)
#   - Run and kill on your nodes any applications you want (using PsTools)
#   - Send your jobs on the Afanasy Renferfarm Manager (http://cgru.info/home)
#   - Copy/Paste files and folders on all clients

# Just change the variables to your own network (hostnames, users and passwords) by editing the RNMconfig.cfg file using a text editor. 
# You can add as many nodes as you need. The limit is your desktop. 

################################## HOW TO INSTALL #####################################

# Install PsTools in C:\ : https://technet.microsoft.com/fr-fr/sysinternals/bb896649.aspx
# Install Anydesk on every computer  : http://anydesk.fr/bureau-a-distance
# Install WakeOnLan on your Workstation  : http://sourceforge.net/projects/aquilawol/

# ATTENTION : add both PSTools and WakeOnLan EXE paths into your Windows environment variables !

# Download the RNM folder in C:\
# Set your private variables by editing RNMconfig.cfg
# Run RNM.pyw
# Enjoy

######################################################################################
################################### SCRIPT BEGIN #####################################
######################################################################################
import tkinter as tk
import sys
import time
import threading

import os
import ntpath
import shutil
from tkinter.filedialog import askopenfilenames, askdirectory

from tkinter import messagebox
from tkinter import PhotoImage
from time import gmtime, strftime

############################################################################
#################### VARIABLES FROM CONFIG FILE ############################
############################################################################

######## WINDOW POSITION AT START ########
import configparser
cfg = configparser.RawConfigParser()
cfgPath = os.path.join(os.path.dirname(__file__))# get relative path
cfg.read(cfgPath + '/RNMconfig.cfg')
posX= cfg.get('Config1', 'window position x')
posY= cfg.get('Config1', 'window position y')

######## PATHS ########
# path to Afanasy folder
afanasy = cfg.get('Config2', 'afanasy')
# path to Afanasy render on clients
pathAFANASY = cfg.get('Config2', 'pathAFANASY')
# path to Anydesk on workstation
pathANYDESK = cfg.get('Config2', 'pathANYDESK')
# path to WakeOnLan on workstation
pathWAKEONLAN = cfg.get('Config2', 'pathWAKEONLAN')
# path to Vray on clients
pathVRAY = cfg.get('Config2', 'pathVRAY')
# config Vray
portVRAY = cfg.get('Config2', 'portVRAY')

######## NUMBER OF CLIENTS ########
NUMCLIENTS = cfg.getint('Nodes', 'number')

######## UI ########
COLORUI = "#17191b"
COLORBUTTON = COLORUI
COLORBUTTONPUSH = COLORUI
FORMBUTTON = "flat"
COLORTEXT = "#3c3c3c"
COLORTEXTSTATUTON = "white"
COLORTEXTSTATUTOFF = "#000000"
COLORTEXTGREEN = "#009fff"
COLORTEXTBLUE = "#009fff"
COLORTEXTYELLOW = "white"
COLORTEXTRED = "red"
COLORTEXTBROWN = "#2b373f"
COLORTEXTTIME = "grey"
FONTEXT="Arial Black"
POLICE= 8

# WINDOW CONFIG
winX = 260
winY = (440+NUMCLIENTS*20)

############################################################################
#############################    UI     ####################################
############################################################################

# MAKE RENDER NODE MANAGER          
ROWS = (NUMCLIENTS+1)
rowLAST = (NUMCLIENTS+2)
rowLAST2 = (NUMCLIENTS+3)
rowSpace = (NUMCLIENTS+4)
rowTEXT = (NUMCLIENTS+5)

class Render_Node_Manager(tk.Tk):
    def __init__(self):
        tk.Tk.__init__(self)

        # ICONS PATH AND NAME
        import os
        os.chdir (cfgPath + "/icons")
        self.WORKSTATIONicon = tk.PhotoImage(file = "workstation-icon.png")
        self.AFSTARTERicon = tk.PhotoImage(file = "afstarter-icon.png")
        self.AFSERVERiconRUN = tk.PhotoImage(file = "afserver-icon-run.png")
        self.AFSERVERiconKILL = tk.PhotoImage(file = "afserver-icon-kill.png")
        self.KEEPERicon = tk.PhotoImage(file = "keeper-icon.png")
        self.AFWATCHicon = tk.PhotoImage(file = "afwatch-icon.png")
        self.DAILIESicon = tk.PhotoImage(file = "dailies-icon.png")
        self.PINGALL = tk.PhotoImage(file = "ping-icon-all.png")
        self.PINGicon = tk.PhotoImage(file = "ping-icon.png")
        self.ADiconOFF = tk.PhotoImage(file = "anydesk-icon-offline.png")
        self.ADiconRUN = tk.PhotoImage(file = "anydesk-icon-run.png")
        self.ADiconRUNALL = tk.PhotoImage(file = "anydesk-icon-runall.png")
        self.VRAYiconOFF = tk.PhotoImage(file = "vray-icon-offline.png")
        self.VRAYiconRUN = tk.PhotoImage(file = "vray-icon-run.png")
        self.VRAYiconKILL = tk.PhotoImage(file = "vray-icon-kill.png")
        self.VRAYiconRUNALL = tk.PhotoImage(file = "vray-icon-runall.png")
        self.VRAYiconKILLALL = tk.PhotoImage(file = "vray-icon-killall.png")
        self.AFiconOFF = tk.PhotoImage(file = "afanasy-icon-offline.png")
        self.AFiconRUN = tk.PhotoImage(file = "afanasy-icon-run.png")
        self.AFiconKILL = tk.PhotoImage(file = "afanasy-icon-kill.png")
        self.AFiconRUN2 = tk.PhotoImage(file = "afanasy-icon-run2.png")
        self.AFiconKILL2 = tk.PhotoImage(file = "afanasy-icon-kill2.png")
        self.AFiconRUNALL = tk.PhotoImage(file = "afanasy-icon-runall.png")
        self.AFiconKILLALL = tk.PhotoImage(file = "afanasy-icon-killall.png")
        self.AFiconRUN2 = tk.PhotoImage(file = "afanasy-icon-run2.png")
        self.AFiconKILL2 = tk.PhotoImage(file = "afanasy-icon-kill2.png")
        self.WOLiconKILL = tk.PhotoImage(file = "wol-icon-kill.png")
        self.WOLiconRUNALL = tk.PhotoImage(file = "wol-icon-runall.png")
        self.WOLiconKILLALL = tk.PhotoImage(file = "wol-icon-killall.png")
        self.WOLiconRUN = tk.PhotoImage(file = "wol-icon-run.png")
        self.WOLiconOFF = tk.PhotoImage(file = "wol-icon-offline.png")
        self.WOLiconKILL = tk.PhotoImage(file = "wol-icon-kill.png")
        self.CPFileIcon = tk.PhotoImage(file = "CPFile-icon.png")
        self.CPFolderIcon = tk.PhotoImage(file = "CPFolder-icon.png")
        self.DeleteFolderIcon = tk.PhotoImage(file = "DeleteFolder-icon.png")
        self.DeleteFileIcon = tk.PhotoImage(file = "DeleteFile-icon.png")
        self.OpenDriveFolderIcon = tk.PhotoImage(file = "OpenDriveFolder-icon.png")
        self.OpenAfanasyFolderIcon = tk.PhotoImage(file = "OpenAfanasyFolder-icon.png")
        self.RemoveTempFolderIcon = tk.PhotoImage(file = "RemoveTempFolder-icon.png")
        self.configIcon = tk.PhotoImage(file = "cfg-icon.png")

        # BUTTONS 
        #row0
        self.b0_01 = list()
        self.b0_01.append(tk.Button(self, image = self.AFSTARTERicon, bg=COLORBUTTON, relief=FORMBUTTON, bd=0, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,command=self.AF_STARTER))
        self.b0_01[-1].grid(row=0, column=0, columnspan=8, pady=5, sticky="s, n, e, w")
        self.b0_01[-1].image = self.AFSTARTERicon
        self.b0_01.append(CreateToolTip2(self.b0_01[-1], ": RUN AFANASY STARTER"))

        # read workstation data from config file
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        
        #row1
        self.b1_01 = list()
        self.b1_01.append(tk.Button(self, image = self.WORKSTATIONicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.DWM_WORKSTATION))
        self.b1_01[-1].grid(row=1, column=0, pady=10)
        self.b1_01[-1].image = self.WORKSTATIONicon
        self.b1_01.append(CreateToolTip2(self.b1_01[-1], ': ' + Wh + " / " + Wu))
        self.b1_02 = list()
        self.b1_02.append(tk.Button(self, image = self.AFWATCHicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_WATCH))
        self.b1_02[-1].grid(row=1, column=1)
        self.b1_02[-1].image = self.AFWATCHicon
        self.b1_02.append(CreateToolTip2(self.b1_02[-1], ": RUN AFANASY WATCH"))
        self.b1_03 = list()
        self.b1_03.append(tk.Button(self, image = self.AFSERVERiconRUN,bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_SERVER))
        self.b1_03[-1].grid(row=1, column=2)
        self.b1_03[-1].image = self.AFSERVERiconRUN
        self.b1_03.append(CreateToolTip2(self.b1_03[-1], ": RUN AFANASY SERVER"))
        self.b1_04 = list()
        self.b1_04.append(tk.Button(self, image = self.AFSERVERiconKILL,bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.KILL_AF_SERVER))
        self.b1_04[-1].grid(row=1, column=3)
        self.b1_04[-1].image = self.AFSERVERiconKILL
        self.b1_04.append(CreateToolTip2(self.b1_04[-1], ": KILL AFANASY SERVER"))
        self.b1_05 = list()
        self.b1_05.append(tk.Button(self, image = self.AFiconRUN2, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_RENDER))
        self.b1_05[-1].grid(row=1, column=4)
        self.b1_05[-1].image = self.AFiconRUN2
        self.b1_05.append(CreateToolTip2(self.b1_05[-1], ": RUN AFANASY RENDER"))
        self.b1_06 = list()
        self.b1_06.append(tk.Button(self, image = self.AFiconKILL2, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.KILL_AF_RENDER))
        self.b1_06[-1].grid(row=1, column=5)
        self.b1_06[-1].image = self.AFiconKILL2
        self.b1_06.append(CreateToolTip2(self.b1_06[-1], ": KILL AFANASY RENDER"))
        self.b1_07 = list()
        self.b1_07.append(tk.Button(self, image = self.DAILIESicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_DAILIES))
        self.b1_07[-1].grid(row=1, column=6)
        self.b1_07[-1].image = self.DAILIESicon
        self.b1_07.append(CreateToolTip2(self.b1_07[-1], ": RUN DAILIES"))
        self.b1_08 = list()
        self.b1_08.append(tk.Button(self, image = self.KEEPERicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_KEEPER))
        self.b1_08[-1].grid(row=1, column=7)
        self.b1_08[-1].image = self.KEEPERicon
        self.b1_08.append(CreateToolTip2(self.b1_08[-1], ": RUN KEEPER"))

        # ROWS CLIENTS
        self.button1 = list()
        self.button1_ttp = list()
        self.button2 = list()
        self.button2_ttp = list()
        self.button3 = list()
        self.button3_ttp = list()
        self.button4 = list()
        self.button4_ttp = list()
        self.button5 = list()
        self.button5_ttp = list()
        self.button6 = list()
        self.button6_ttp = list()
        self.button7 = list()
        self.button7_ttp = list()
        self.button8 = list()
        self.button8_ttp = list()

        for i in range (1,ROWS):
            j= i+1
            RN = ("RN"+str(i))
            Node = ("Node"+str(i))
            # get hostname, user and password from config file
            import configparser
            cfg = configparser.RawConfigParser()
            cfg.read(cfgPath + '/RNMconfig.cfg')
            h = cfg.get(Node, 'hostname')
            u = cfg.get(Node, 'user')
            p = cfg.get(Node, 'password')
            
            self.button1.append(tk.Button(text=RN, font=(FONTEXT, POLICE), foreground=COLORTEXT, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command1(hostname, user, password, rendernode)))
            self.button1[-1].grid(row=j, column=0)
            self.button1_ttp.append(CreateToolTip(self.button1[-1], h + " / " + u))
            
            self.button2.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command2(hostname, user, password, rendernode)))
            self.button2[-1].grid(row=j, column=1)
            self.button2[-1].image = self.PINGicon
            self.button2_ttp.append(CreateToolTip2(self.button2[-1], ": RUN ANYDESK / " + h))

            self.button3.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command3(hostname, user, password, rendernode)))
            self.button3[-1].grid(row=j, column=2)
            self.button3[-1].image = self.PINGicon
            self.button3_ttp.append(CreateToolTip2(self.button3[-1], ": RUN VRAY / " + h))

            self.button4.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command4(hostname, user, password, rendernode)))
            self.button4[-1].grid(row=j, column=3)
            self.button4[-1].image = self.PINGicon
            self.button4_ttp.append(CreateToolTip2(self.button4[-1], ": KILL VRAY / " + h))

            self.button5.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command5(hostname, user, password, rendernode)))
            self.button5[-1].grid(row=j, column=4)
            self.button5[-1].image = self.PINGicon
            self.button5_ttp.append(CreateToolTip2(self.button5[-1], ": RUN AFANASY / " + u))

            self.button6.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command6(hostname, user, password, rendernode)))
            self.button6[-1].grid(row=j, column=5)
            self.button6[-1].image = self.PINGicon
            self.button6_ttp.append(CreateToolTip2(self.button6[-1], ": KILL AFANASY / " + h))

            self.button7.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command7(hostname, user, password, rendernode)))
            self.button7[-1].grid(row=j, column=6)
            self.button7[-1].image = self.PINGicon
            self.button7_ttp.append(CreateToolTip2(self.button7[-1], ": WAKE ON LAN / " + h))

            self.button8.append(tk.Button(image = self.PINGicon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH,
                                     command= lambda hostname= h, user= u, password= p, rendernode= RN : self.command8(hostname, user, password, rendernode)))
            self.button8[-1].grid(row=j, column=7)
            self.button8[-1].image = self.PINGicon
            self.button8_ttp.append(CreateToolTip2(self.button8[-1], ": SHUTDOWN / " + h))
            
        #LAST ROW
        self.buttonLAST_01 = list()
        self.buttonLAST_01.append(tk.Button(self, image = self.PINGALL, bg=COLORUI, relief=FORMBUTTON, height = 20, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.PING_ALL))
        self.buttonLAST_01[-1].grid(row=rowLAST, column=0, pady=10)
        self.buttonLAST_01[-1].image = self.PINGALL
        self.buttonLAST_01.append(CreateToolTip2(self.buttonLAST_01[-1], ": PING RENDERFARM "))
        self.buttonLAST_02 = list()
        self.buttonLAST_02.append(tk.Button(self, image = self.ADiconRUNALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AD_LCH))
        self.buttonLAST_02[-1].grid(row=rowLAST, column=1)
        self.buttonLAST_02[-1].image = self.ADiconRUNALL
        self.buttonLAST_02.append(CreateToolTip2(self.buttonLAST_02[-1], ": RUN ANYDESK / RENDERFARM "))
        self.buttonLAST_03 = list()
        self.buttonLAST_03.append(tk.Button(self, image = self.VRAYiconRUNALL, bg=COLORUI, relief=FORMBUTTON,activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.VRAY3_LAUNCH_THEM_ALL))
        self.buttonLAST_03[-1].grid(row=rowLAST, column=2)
        self.buttonLAST_03[-1].image = self.VRAYiconRUNALL
        self.buttonLAST_03.append(CreateToolTip2(self.buttonLAST_03[-1], ": RUN VRAY / RENDERFARM "))
        self.buttonLAST_04 = list()
        self.buttonLAST_04.append(tk.Button(self, image = self.VRAYiconKILLALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.VRAY3_KILL_THEM_ALL))
        self.buttonLAST_04[-1].grid(row=rowLAST, column=3)
        self.buttonLAST_04[-1].image = self.VRAYiconKILLALL
        self.buttonLAST_04.append(CreateToolTip2(self.buttonLAST_04[-1], ": KILL VRAY / RENDERFARM "))
        self.buttonLAST_05 = list()
        self.buttonLAST_05.append(tk.Button(self, image = self.AFiconRUNALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_LAUNCH_THEM_ALL))
        self.buttonLAST_05[-1].grid(row=rowLAST, column=4)
        self.buttonLAST_05[-1].image = self.AFiconRUNALL
        self.buttonLAST_05.append(CreateToolTip2(self.buttonLAST_05[-1], ": RUN AFANASY / RENDERFARM "))
        self.buttonLAST_06 = list()
        self.buttonLAST_06.append(tk.Button(self, image = self.AFiconKILLALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.AF_KILL_THEM_ALL))
        self.buttonLAST_06[-1].grid(row=rowLAST, column=5)
        self.buttonLAST_06[-1].image = self.AFiconKILLALL
        self.buttonLAST_06.append(CreateToolTip2(self.buttonLAST_06[-1], ": KILL AFANASY / RENDERFARM "))
        self.buttonLAST_07 = list()
        self.buttonLAST_07.append(tk.Button(self, image = self.WOLiconRUNALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.WOL_WAKE_THEM_ALL))
        self.buttonLAST_07[-1].grid(row=rowLAST, column=6)
        self.buttonLAST_07[-1].image = self.WOLiconRUNALL
        self.buttonLAST_07.append(CreateToolTip2(self.buttonLAST_07[-1], ": WAKE ON LAN / RENDERFARM "))
        self.buttonLAST_08 = list()
        self.buttonLAST_08.append(tk.Button(self, image = self.WOLiconKILLALL, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.WOL_KILL_THEM_ALL))
        self.buttonLAST_08[-1].grid(row=rowLAST, column=7)
        self.buttonLAST_08[-1].image = self.WOLiconKILLALL
        self.buttonLAST_08.append(CreateToolTip2(self.buttonLAST_08[-1], ": SHUTDOWN / RENDERFARM "))
      
        #LAST ROW 2
        self.buttonLAST2_00 = list()
        self.buttonLAST2_00.append(tk.Button(self, image = self.OpenDriveFolderIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.OpenDriveFolder))
        self.buttonLAST2_00[-1].grid(row=rowLAST2, column=0, sticky="n")
        self.buttonLAST2_00[-1].image = self.OpenDriveFolderIcon
        self.buttonLAST2_00.append(CreateToolTip2(self.buttonLAST2_00[-1], ": OPEN Drive FOLDER / WORKSTATION "))
        self.buttonLAST2_01 = list()
        self.buttonLAST2_01.append(tk.Button(self, image = self.OpenAfanasyFolderIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.OpenAfanasyFolder))
        self.buttonLAST2_01[-1].grid(row=rowLAST2, column=6)
        self.buttonLAST2_01[-1].image = self.OpenAfanasyFolderIcon
        self.buttonLAST2_01.append(CreateToolTip2(self.buttonLAST2_01[-1], ": OPEN Afanasy FOLDER / WORKSTATION "))
        self.buttonLAST2_02 = list()
        self.buttonLAST2_02.append(tk.Button(self, image = self.CPFileIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.CopyPasteFile))
        self.buttonLAST2_02[-1].grid(row=rowLAST2, column=2)
        self.buttonLAST2_02[-1].image = self.CPFileIcon
        self.buttonLAST2_02.append(CreateToolTip2(self.buttonLAST2_02[-1], ": CLONE FILE(s) / RENDERFARM "))
        self.buttonLAST2_03 = list()
        self.buttonLAST2_03.append(tk.Button(self, image = self.CPFolderIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.CopyPasteFolder))
        self.buttonLAST2_03[-1].grid(row=rowLAST2, column=1)
        self.buttonLAST2_03[-1].image = self.CPFolderIcon
        self.buttonLAST2_03.append(CreateToolTip2(self.buttonLAST2_03[-1], ": CLONE FOLDER / RENDERFARM "))
        self.buttonLAST2_04 = list()
        self.buttonLAST2_04.append(tk.Button(self, image = self.DeleteFolderIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.DeleteFolder))
        self.buttonLAST2_04[-1].grid(row=rowLAST2, column=3)
        self.buttonLAST2_04[-1].image = self.DeleteFolderIcon
        self.buttonLAST2_04.append(CreateToolTip2(self.buttonLAST2_04[-1], ": DELETE FOLDER / RENDERFARM "))
        self.buttonLAST2_05 = list()
        self.buttonLAST2_05.append(tk.Button(self, image = self.DeleteFileIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.DeleteFile))
        self.buttonLAST2_05[-1].grid(row=rowLAST2, column=4)
        self.buttonLAST2_05[-1].image = self.DeleteFileIcon
        self.buttonLAST2_05.append(CreateToolTip2(self.buttonLAST2_05[-1], ": DELETE File(s) / RENDERFARM "))
        self.buttonLAST2_06 = list()
        self.buttonLAST2_06.append(tk.Button(self, image = self.RemoveTempFolderIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.RemoveTempFolder))
        self.buttonLAST2_06[-1].grid(row=rowLAST2, column=5)
        self.buttonLAST2_06[-1].image = self.RemoveTempFolderIcon
        self.buttonLAST2_06.append(CreateToolTip2(self.buttonLAST2_06[-1], ": REMOVE  '_RNMtemp'  FOLDER "))
        self.buttonLAST2_07 = list()
        self.buttonLAST2_07.append(tk.Button(self, image = self.configIcon, bg=COLORUI, relief=FORMBUTTON, activebackground=COLORBUTTONPUSH, activeforeground=COLORBUTTONPUSH, command=self.openConfig))
        self.buttonLAST2_07[-1].grid(row=rowLAST2, column=7)
        self.buttonLAST2_07[-1].image = self.configIcon
        self.buttonLAST2_07.append(CreateToolTip2(self.buttonLAST2_07[-1], ": OPEN CONFIG "))
        
        # OUTPUT TEXTS
        self.text = tk.Text(self, wrap="word", bg=COLORBUTTON, bd=0, font=("Arial", 7))
        self.text.grid(row=rowTEXT, columnspan=8, pady=20, sticky="s, n, e, w")
        self.text.tag_configure("stdout", font=("Arial", 10), justify="right", foreground="white")
        self.text.tag_configure("stderr1", justify="right", foreground="white")
        self.text.tag_configure("stderr2", justify="right", foreground=COLORTEXTRED)
        self.text.tag_configure("stderr3", justify="left", foreground=COLORTEXTTIME)
        self.text.tag_configure("stderr4", justify="right", foreground=COLORTEXTGREEN)
        self.text.tag_configure("stderr5", justify="right", foreground=COLORTEXTYELLOW)
        self.text.tag_configure("stderr6", justify="right", foreground=COLORTEXTBLUE)
        self.text.tag_configure("stderr7", justify="right", foreground=COLORTEXTBROWN)
        sys.stdout = TextRedirector(self.text, "stdout")
        sys.stderr1 = TextRedirector(self.text, "stderr1")
        sys.stderr2 = TextRedirector(self.text, "stderr2")
        sys.stderr3 = TextRedirector(self.text, "stderr3")
        sys.stderr4 = TextRedirector(self.text, "stderr4")
        sys.stderr5 = TextRedirector(self.text, "stderr5")
        sys.stderr6 = TextRedirector(self.text, "stderr6")
        sys.stderr7 = TextRedirector(self.text, "stderr7")

    # PING NETWORK IN LOOP 
        thread = threading.Thread(target=self.ping, args=())
        thread.daemon = True
        thread.start() 
    def ping (self):
        for i in range(0,NUMCLIENTS):
            j=i+1
            # get hostname from config file
            Node = ("Node"+str(j))
            import configparser
            cfg = configparser.RawConfigParser()
            cfg.read(cfgPath + '/RNMconfig.cfg')
            hostname = cfg.get(Node, 'hostname')

            from subprocess import Popen, PIPE
            cmd = ['ping', hostname, '-t', '-n', '1', '-i', '1']
            process = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()
            if (process.returncode == 0):
                B1 = self.button1[i] 
                B1 ['foreground'] = COLORTEXTSTATUTON
                B2 = self.button2[i]
                B2 ['image'] = self.ADiconRUN
                self.button2[-1].image = self.ADiconRUN
                B3 = self.button3[i]
                B3 ['image'] = self.VRAYiconRUN
                self.button3[-1].image = self.VRAYiconRUN
                B4 = self.button4[i]
                B4 ['image'] = self.VRAYiconKILL
                self.button4[-1].image = self.VRAYiconKILL
                B5 = self.button5[i]
                B5 ['image'] = self.AFiconRUN
                self.button5[-1].image = self.AFiconRUN
                B6 = self.button6[i]
                B6 ['image'] = self.AFiconKILL
                self.button6[-1].image = self.AFiconKILL
                B7 = self.button7[i]
                B7 ['image'] = self.WOLiconOFF
                self.button7[-1].image = self.WOLiconOFF
                B8 = self.button8[i]
                B8 ['image'] = self.WOLiconKILL
                self.button8[-1].image = self.WOLiconKILL
            else:
                B1 = self.button1[i] 
                B1 ['foreground'] = COLORTEXTSTATUTOFF
                B2 = self.button2[i]
                B2 ['image'] = self.ADiconOFF
                self.button2[-1].image = self.ADiconOFF
                B3 = self.button3[i]
                B3 ['image'] = self.VRAYiconOFF
                self.button3[-1].image = self.VRAYiconOFF
                B4 = self.button4[i]
                B4 ['image'] = self.VRAYiconOFF
                self.button4[-1].image = self.VRAYiconOFF
                B5 = self.button5[i]
                B5 ['image'] = self.AFiconOFF
                self.button5[-1].image = self.AFiconOFF
                B6 = self.button6[i]
                B6 ['image'] = self.AFiconOFF
                self.button6[-1].image = self.AFiconOFF
                B7 = self.button7[i]
                B7 ['image'] = self.WOLiconRUN
                self.button7[-1].image = self.WOLiconRUN
                B8 = self.button8[i]
                B8 ['image'] = self.WOLiconOFF
                self.button8[-1].image = self.WOLiconOFF            
        self.after(60000, self.ping) # SET TIME BETWEEN PINGS

        
    ################## COMMANDS ##################
    # ROW 0 (SEND JOB)
    def AF_STARTER(self): #LAUNCH AFANASY STARTER
        import os
        os.chdir (afanasy + "/start/AFANASY")
        os.startfile("afstarter.cmd")
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr5.write(" Afanasy starter launched\n") 
    # ROW 1  (WORKSTATION)
    def DWM_WORKSTATION(self): #OPEN WORKSTATION TASK MANAGER
        import os
        os.chdir ("C:/Windows/System32")
        os.startfile("taskmgr.exe")
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        import subprocess
        subprocess.Popen (r'"psexec" \\{} -u {} -p {} cmd /k ipconfig'.format(Wh, Wu, Wp), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))        # read workstation data from config file
        sys.stderr1.write(" Workstation task manager launched\n") 
    def AF_SERVER(self): #LAUNCH AFANASY SERVER
        # read workstation data from config file
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        import subprocess
        subprocess.Popen ('psexec "{}/start/AFANASY/_afserver.cmd" \\{} -i -d -u {} -p {} '.format(afanasy, Wh, Wu, Wp), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr4.write(" Afanasy server running\n")
    def KILL_AF_SERVER(self): #KILL AFANASY SERVER
        if messagebox.askokcancel("WARNING", "KILL AF SERVER?", icon="warning"):
            import subprocess
            subprocess.Popen (r'taskkill /f /im afserver.exe', shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write(" Afanasy server killed\n") 
    def AF_WATCH(self): #LAUNCH AFANASY WATCH
        import os
        os.chdir (afanasy + "/start/AFANASY")
        os.startfile("afwatch.cmd")
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr6.write(" Afanasy watch running\n") 
    def AF_RENDER(self): #RUN AFANASY RENDER
        # read workstation data from config file
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        import subprocess
        subprocess.Popen ('psexec "{}/start/AFANASY/render.cmd" \\{} -i -d -u {} -p {} '.format(afanasy, Wh, Wu, Wp), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr4.write(" Afanasy render running\n")
    def KILL_AF_RENDER(self): #KILL AFANASY RENDER
        if messagebox.askokcancel("WARNING", "KILL AF RENDER?", icon="warning"):
            import subprocess
            subprocess.Popen (r'taskkill /f /im afrender.exe', shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write(" Afanasy render killed\n") 
    def AF_DAILIES(self): #LAUNCH AFANASY DAILIES
        import os
        os.chdir (afanasy + "/start/MovieMaker")
        os.startfile("dailies.cmd")
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write(" Afanasy daillies launched\n")
    def AF_KEEPER(self): #LAUNCH AFANASY KEEPER
        import os
        os.chdir (afanasy)
        os.startfile("start.cmd")
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write(" Afanasy keeper running\n")

    # ROWS COMMANDS (LAMBDA)
    def command1(self, hostname, user, password, rendernode): #PSLIST CLIENT
        import subprocess
        subprocess.Popen (r'"SC" \\{} start RemoteRegistry ' .format(hostname, user, password), shell=True)
        subprocess.Popen (r'"pslist" \\{} -u {} -p {} -m -s' .format(hostname, user, password), shell=False)
        subprocess.Popen (r'"psexec" \\{} -u {} -p {} cmd /k ipconfig/all'.format(hostname, user, password), shell=False)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write(" PSLIST " + rendernode + " running\n")       
    def command2(self, hostname, user, password, rendernode): #LAUNCH ANYDESK VIEWER
        import subprocess
        subprocess.Popen (r'{} {}' .format(pathANYDESK, hostname), shell=True)
        subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p  {} "C:\Windows\System32\taskmgr.exe"' .format(hostname, user, password), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr6.write(" AnyDesk viewer " + rendernode + " running\n")           
    def command3(self, hostname, user, password, rendernode): #RUN VRAY STANDALONE SLAVE
        import subprocess
        subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} {} -server -portNumber={}' .format(hostname, user, password, pathVRAY, portVRAY), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr4.write(" VRay render slave " + rendernode + " running\n")
    def command4(self, hostname, user, password, rendernode): #KILL VRAY STANDALONE SLAVE
        if messagebox.askokcancel("WARNING", "KILL VRAY " + rendernode + " ?", icon="warning"):
            import subprocess
            subprocess.Popen (r'"pskill" -t \\{} -u {} -p {} vray.exe' .format(hostname, user, password), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write(" VRayRenderSlave " + rendernode + " killed\n")             
    def command5(self, hostname, user, password, rendernode): #RUN AFANASY RENDER      
        import subprocess
        subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p  {} {}' .format(hostname, user, password, pathAFANASY), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr4.write(" Afanasy render " + rendernode + " running\n")      
    def command6(self, hostname, user, password, rendernode): #KILL AFANASY RENDER
        if messagebox.askokcancel("WARNING", "KILL AF " + rendernode + " ?", icon="warning"):
            import subprocess
            subprocess.Popen (r'"pskill" -t \\{} -u {} -p {} afrender.exe' .format(hostname, user, password), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write(" Afanasy render " + rendernode + " killed\n")
    def command7(self, hostname, user, password, rendernode): #WAKE ON LAN CLIENT
        import subprocess
        subprocess.Popen (r'{} -w -m {} -f' .format(pathWAKEONLAN, hostname), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr4.write(" " + rendernode + " waking up\n")
    def command8(self, hostname, user, password, rendernode): #SHUTDOWN CLIENT
            if messagebox.askokcancel("WARNING", "SHUTDOWN " + rendernode + " ?", icon="warning"):
                import subprocess
                subprocess.Popen (r'{} -s -m {} -t 01 -f' .format(pathWAKEONLAN, hostname), shell=True)
                sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
                sys.stderr2.write(" " + rendernode + " shutdowned\n")
                
    # LAST ROW COMMANDS
    def PING_ALL(self): #PING ALL CLIENTS AND OUTPUT STATE
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S\n"))
        for i in range (1,ROWS):
            # get hostname from config file
            Node = ("Node"+str(i))
            import configparser
            cfg = configparser.RawConfigParser()
            cfg.read(cfgPath + '/RNMconfig.cfg')
            hostname = cfg.get(Node, 'hostname')
            # ping network
            from subprocess import Popen, PIPE
            cmd = ['ping', hostname, '-t', '-n', '1', '-i', '1']
            process = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()
            if (process.returncode == 0):
                sys.stderr6.write (hostname + " IS ONLINE\n")
            else:
                sys.stderr2.write (hostname + " IS OFFLINE\n")
    def AD_LCH(self): #LAUNCH ANYDESK ON EVERY CLIENTS
        import subprocess
        for i in range (1,ROWS):
            # get hostname, user and password from config file            
            Node = ("Node"+str(i))
            import configparser
            cfg = configparser.RawConfigParser()
            cfg.read(cfgPath + '/RNMconfig.cfg')
            h = cfg.get(Node, 'hostname')
            u = cfg.get(Node, 'user')
            p = cfg.get(Node, 'password')
            subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p  {} {}' .format(h, u, p, pathANYDESK), shell=True)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr5.write(" LAUNCH ALL ANYDESK\n")
    def VRAY3_LAUNCH_THEM_ALL(self): #LAUNCH VRAY STANDALONE ON EVERY CLIENTS
        if messagebox.askokcancel("WARNING", "LAUNCH ALL VRAY RENDERSLAVES ?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} {} -server -portNumber={}' .format(h, u, p, pathVRAY, portVRAY), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr5.write(" VRAY SLAVES RUNNING\n")
    def VRAY3_KILL_THEM_ALL(self): #KILL VRAY STANDALONE ON EVERY CLIENTS
        if messagebox.askokcancel("WARNING", "KILL ALL VRAY RENDERSLAVES ?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                subprocess.Popen (r'"pskill" -t \\{} -u {} -p {} vray.exe' .format(h, u, p), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr5.write(" VRAY RENDER SLAVES KILLED\n")
    def AF_LAUNCH_THEM_ALL(self): #LAUNCH AFANASY RENDERS ON EVERY CLIENTS
        if messagebox.askokcancel("WARNING", "LAUNCH ALL AF RENDERS?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p  {} {}' .format(h, u, p, pathAFANASY), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr5.write(" AFANASY RENDERS RUNNING\n")
    def AF_KILL_THEM_ALL(self): #KILL AFANASY RENDERS ON EVERY CLIENTS
        if messagebox.askokcancel("WARNING", "KILL ALL AFANASY RENDERS?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                subprocess.Popen (r'"pskill" -t \\{} -u {} -p {} afrender.exe' .format(h, u, p), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr7.write(" AFANASY RENDERS KILLED\n")
    def WOL_WAKE_THEM_ALL(self): #WAKE ON LAN ALL CLIENTS
        if messagebox.askokcancel("WARNING", "WAKE ALL NODES?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                subprocess.Popen (r'{} -w -m {} -f' .format(pathWAKEONLAN, h), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr5.write(" RENDERFARM WAKING UP\n")
    def WOL_KILL_THEM_ALL(self): #ALL CLIENTS SHUTDOWN
        if messagebox.askokcancel("WARNING", "SHUTDOWN ALL NODES?", icon="warning"):
            import subprocess
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                subprocess.Popen (r'{} -s -m {} -t 01 -f' .format(pathWAKEONLAN, h), shell=True)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr7.write(" RENDERFARM SHUTDOWNED\n")

    # LAST ROW2 COMMANDS
    
    #OPEN DRIVE FOLDER   
    def OpenDriveFolder(self):
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        driveNameFolder = cfg.get('Config2', 'driveNameFolder')
        drivePath = ('\\' + '\\' + Wh + '\\' + driveNameFolder)
        import os
        os.startfile(drivePath)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write(" "+ drivePath)
        sys.stderr3.write(" OPENED\n")
    #OPEN AFANASY FOLDER   
    def OpenAfanasyFolder(self):
        import os
        os.startfile(afanasy)
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write("  AFANASY FOLDER OPENED\n")
    # DELETE FOLDER 
    def DeleteFolder(self) :
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wu = cfg.get('Workstation', 'user')
        import subprocess
        folderPath = askdirectory(title = "DELETE FOLDER (path cloned to renderfarm)") #CHOOSE FOLDER
        if folderPath :
            folderPath = '"' + '\\'.join(folderPath.split('/')) + '"'
            targetStderr = "...".join(folderPath.split(Wu))
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                target = folderPath
                if Wu in target : #CHANGE USER
                    target = u.join(target.split(Wu))   
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c rmdir /S /Q {}'.format(h, u, p, target), shell=True) #COPY FOLDER
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr1.write(" \n" + targetStderr + "\n")
            sys.stderr2.write(" FOLDER DELETED FROM RENDERFARM\n")
        else :
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write("  Delete folder canceled \n")
            
    # DELETE FILE(s) 
    def DeleteFile(self) :
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wu = cfg.get('Workstation', 'user')
        import subprocess
        filenames = askopenfilenames(title = "DELETE FILE (path cloned to renderfarm") #CHOOSE FILE(s)
        if filenames :
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                for item in filenames : # COPY FILES 
                    item = '"' + '\\'.join(item.split('/')).upper() +'"'
                    if Wu in item : #CHANGE USER
                        item = u.join(item.split(Wu)).upper()
                    import subprocess   
                    subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c del {}'.format(h, u, p, item), shell=True) #COPY FILE /TEMP      
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            for item in filenames : # COPY FILES 
                path,fileName = os.path.split(item)
                sys.stderr1.write( "\n" + fileName)
            sys.stderr2.write("\n FILE(S) DELETED FROM RENDERFARM\n" )
        else :
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write("  Delete file(s) canceled \n")
        
    # CLONE FILE(s) 
    def CopyPasteFile(self) :
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        driveNameFolder = cfg.get('Config2', 'driveNameFolder')
        tempFiles = "tempFiles_" + strftime("%Y-%m-%d_%H-%M-%S")
        tempTarget = ('"\\' + '\\' + Wh + '\\' + driveNameFolder + '\\_RNMtemp\\' + tempFiles + '\\"' )
        tempDirectory = ('"//' + Wh + '/' + driveNameFolder + '/_RNMtemp/' + tempFiles + '"' )
        tempFilenames = '\\'.join(tempDirectory.split('/')).upper()
        import subprocess
        filenames = askopenfilenames(title = "CLONE FILE(s) ON RENDERFARM") #CHOOSE FILE(s)   
        if filenames :
            for item in filenames : # COPY FILES 
                item = '"' + '\\'.join(item.split('/')).upper() + '"'
                import subprocess
                subprocess.Popen (r'"xcopy" /Q /Y /C /H /K {} {}'.format(item, tempTarget), shell=True) #COPY FILE /TEMP
            import os.path
            folderPath = os.path.dirname(filenames[0])
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                target = '"' + '\\'.join(folderPath.split('/')) + '\\"'
                if Wu in target : #CHANGE USER
                    target = u.join(target.split(Wu))
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c mkdir {}'.format(h, u, p, target), shell=True) #CREATE FOLDER
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c xcopy /Q /Y {} {}'.format(h, u, p, tempFilenames, target), shell=True) #COPY FILES
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            for item in filenames : # COPY FILES
                import os.path
                path,fileName = os.path.split(item)
                sys.stderr1.write( "\n" + fileName)
            sys.stderr4.write(" \n destination folder :\n" )
            sys.stderr1.write(folderPath + "\n")
            sys.stderr6.write("FILE(S) CLONED ON RENDERFARM\n")
        else:
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write("  Copy file canceled \n")

    # CLONE FOLDER
    def CopyPasteFolder(self) :
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        Wu = cfg.get('Workstation', 'user')
        Wp = cfg.get('Workstation', 'password')
        driveNameFolder = cfg.get('Config2', 'driveNameFolder')
        import subprocess
        folderSelect = askdirectory(title = "CLONE FOLDER ON RENDERFARM") #CHOOSE FOLDER
        if folderSelect :
            folderSelectTemp = '"' + '\\'.join(folderSelect.split('/')) +'"'
            path,folderName = os.path.split(folderSelect)
            tempTarget = ('"\\' + '\\' + Wh + '\\' + driveNameFolder + '\\_RNMtemp\\' + folderName + '\"')
            tempDirectory = ('//' + Wh + '/' + driveNameFolder + '/_RNMtemp/' + folderName)
            tempFolder = '"' + '\\'.join(tempDirectory.split('/')) + '"'
            subprocess.Popen (r'"xcopy" /C /Y /Q /E /I /H /K {} {}'.format(folderSelectTemp, tempTarget), shell=True) #COPY FOLDER /TEMP
            for i in range (1,ROWS):
                # get hostname, user and password from config file            
                Node = ("Node"+str(i))
                import configparser
                cfg = configparser.RawConfigParser()
                cfg.read(cfgPath + '/RNMconfig.cfg')
                h = cfg.get(Node, 'hostname')
                u = cfg.get(Node, 'user')
                p = cfg.get(Node, 'password')
                target = folderSelect
                if Wu in target : #CHANGE USER
                    target = u.join(target.split(Wu))
                target = '"' + '\\'.join(target.split('/')) + '\\"'            
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c mkdir {}'.format(h, u, p, target), shell=True) #CREATE FOLDER
                subprocess.Popen (r'"psexec" \\{} -i -h -d -u {} -p {} cmd /c xcopy /I /Q /E /Y /H /K {} {}'.format(h, u, p, tempFolder, target), shell=True) #COPY FOLDER        
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr1.write("\n " + folderName)
            sys.stderr6.write("\n FOLDER CLONED ON RENDERFARM \n")
        else:
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write("  Copy folder canceled \n")
            
    # REMOVE TEMP FOLDER
    def RemoveTempFolder(self) :
        import configparser
        cfg = configparser.RawConfigParser()
        cfg.read(cfgPath + '/RNMconfig.cfg')
        Wh = cfg.get('Workstation', 'hostname')
        driveNameFolder = cfg.get('Config2', 'driveNameFolder')
        tempDirectory = ('//' + Wh + '/' + driveNameFolder + '/_RNMtemp')
        output = os.path.isdir(tempDirectory)
        if output == True :
            shutil.rmtree(tempDirectory)
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr2.write("  TEMP FOLDER DELETED \n")
        else :
            sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
            sys.stderr1.write("  '_RNMtemp' DOESN'T EXIST \n")
            
    # OPEN CONFIG
    def openConfig(self) :
        configFile = ( cfgPath + '\RNMconfig.cfg')
        import subprocess
        subprocess.Popen([r"C:\Windows\system32\notepad.exe", configFile])
        sys.stderr3.write (strftime("%Y-%m-%d %H:%M:%S"))
        sys.stderr1.write("  CONFIG FILE OPENED \n")

##################### OUTPUT UI ####################
class TextRedirector(object):
    def __init__(self, widget, tag="stdout"):
        self.widget = widget
        self.tag = tag
    def write(self, str):
        self.widget.configure(state="normal")
        self.widget.insert("end", str, (self.tag,))
        self.widget.yview("end")
        self.widget.configure(state="disabled")
        
####################################################
####################################################
# MAKE TOOLTIP 1
class CreateToolTip(object):
    def __init__(self, widget, text='widget info'):
        self.widget = widget
        self.text = text
        self.widget.bind("<Enter>", self.enter)
        self.widget.bind("<Leave>", self.close)
    def enter(self, event=None):
        x = y = 0
        x, y, cx, cy = self.widget.bbox("insert")
        x += self.widget.winfo_rootx() + 35 # tooltip position X
        y += self.widget.winfo_rooty() + 15 # tooltip position Y
        self.tw = tk.Toplevel(self.widget)
        self.tw.wm_overrideredirect(True)
        self.tw.wm_geometry("+%d+%d" % (x, y))
        # config text
        label = tk.Label(self.tw, text=self.text, justify='left', foreground= COLORTEXTYELLOW, background=COLORUI, relief='flat', borderwidth=0, font=("arial", "8", "bold"))
        label.pack(ipadx=1)
    def close(self, event=None):
        if self.tw:
            self.tw.destroy()
# MAKE TOOLTIP 2
class CreateToolTip2(object):
    def __init__(self, widget, text='widget info'):
        self.widget = widget
        self.text = text
        self.widget.bind("<Enter>", self.enter)
        self.widget.bind("<Leave>", self.close)
    def enter(self, event=None):
        x = y = 0
        x, y, cx, cy = self.widget.bbox("insert")
        x += app.winfo_x() + 20 # tooltip position X
        y += app.winfo_y() + winY -210  # tooltip position Y
        self.tw = tk.Toplevel(self.widget)
        self.tw.wm_overrideredirect(True)
        self.tw.wm_geometry("+%d+%d" % (x, y))
        # config text
        label = tk.Label(self.tw, text=self.text, justify='right' , foreground= "grey", background=COLORUI, relief='flat', borderwidth=0, font=("arial", "8", "bold"))
        label.pack(ipadx=1)
    def close(self, event=None):
        if self.tw:
            self.tw.destroy()
            
# WINDOW POSITION ON CLOSING 
def on_closing():
    import configparser
    cfg = configparser.RawConfigParser()
    cfg.read(cfgPath + '/RNMconfig.cfg')
    cfg.set('Config1', 'window position x', (app.winfo_x()))
    cfg.set('Config1', 'window position y', (app.winfo_y()))
    cfg.write(open(cfgPath + '/RNMconfig.cfg','w'))
    app.destroy()
    
####################################################
app = Render_Node_Manager()
app.iconbitmap (cfgPath + "/icons/RNM.ico")
app.title ("Render Node Manager") #WINDOW TITLE
# ROWS CONFIG
app.rowconfigure(0, minsize = 60, weight=1)
app.rowconfigure(1, minsize = 50, weight=1)
for i in range (2,rowLAST):
    app.rowconfigure(i, minsize = 20,  weight=1)
app.rowconfigure(rowLAST, minsize = 50, weight=1)
app.rowconfigure(rowLAST2, minsize = 25, weight=1)
app.rowconfigure(rowSpace, minsize = 15, weight=1)
app.rowconfigure(rowTEXT, minsize = 250, weight=10)
# COLUMNS CONFIG
app.configure (bd=10, bg=COLORUI)
app.columnconfigure(0, weight=1)
app.columnconfigure(1, weight=1)
app.columnconfigure(2, weight=1)
app.columnconfigure(3, weight=1)
app.columnconfigure(4, weight=1)
app.columnconfigure(5, weight=1)
app.columnconfigure(6, weight=1)
app.columnconfigure(7, weight=1)
app.columnconfigure(8, weight=1)

WINgeometry = (str(winX)+"x"+str(winY)+"+"+str(posX)+"+"+str(posY))
app.geometry (WINgeometry)

app.resizable(width = False, height = True)

app.protocol("WM_DELETE_WINDOW", on_closing)
app.mainloop()

#############################################################################
########################### END  OF SCRIPT ##################################
#############################################################################
# JR 
